<?php

namespace FiloBlu\AttributeRelated\Block;

use Magento\Catalog\Block\Product\ImageBuilder;
use Magento\Catalog\Helper\Image;
use Magento\Catalog\Model\Product\Attribute\Repository;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\CatalogInventory\Helper\Stock;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Registry;
use Magento\Framework\View\Element\Template;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Swatches\Helper\Media;
use Magento\Swatches\Model\ResourceModel\Swatch\CollectionFactory as SwatchCollectionFactory;
use Magento\Swatches\Model\Swatch;

/**
 * Class AttributeRelated
 * @package FiloBlu\AttributeRelated\Block
 */
class AttributeRelated extends Template
{
    /**
     * @var string
     */
    const ALLOWED_PRODUCT_TYPE_PATH = 'filoblu_attributerelated/general/allowed_product_types';
    /**
     * @var string
     */
    const PRODUCT_LIMIT_PATH = 'filoblu_attributerelated/general/product_limit';
    /**
     * @var string
     */
    const ENABLED_ATTRIBUTE_RELATED_PATH = 'filoblu_attributerelated/general/enabled_attribute_related';
    /**
     * @var string
     */
    const ENABLED_ATTRIBUTE_RELATED_LISTING_PATH = 'filoblu_attributerelated/general/enabled_attribute_related_listing';
    /**
     * @var string
     */
    const ATTRIBUTE_LABEL_PATH = 'filoblu_attributerelated/general/swatch_attribute_label';
    /**
     * @var string
     */
    const SWATCH_ATTRIBUTE_PATH = 'filoblu_attributerelated/general/swatch_attribute';
    /**
     * @var string
     */
    const ATTRIBUTE_RELATION_PATH = 'filoblu_attributerelated/general/attribute_relation';
    /**
     * @var string
     */
    const HIDE_OUT_OF_STOCK_PATH = 'filoblu_attributerelated/general/hide_out_of_stock';
    /**
     * @var
     */
    protected $currentProduct;

    /**
     * @var StoreManagerInterface
     */
    protected $imageBuilder;
    /**
     * @var Registry
     */
    protected $registry;
    /**
     * @var CollectionFactory
     */
    protected $productCollectionFactory;
    /**
     * @var Stock
     */
    protected $stockFilter;
    /**
     * @var Media
     */
    protected $swatchHelperMedia;
    /**
     * @var array
     */
    protected $lookupWatch = [];
    /**
     * @var Image|mixed
     */
    protected $imageHelper;
    /**
     * @var string
     */
    protected $defaultSwatchThumb;
    /**
     * @var string
     */
    protected $defaultSwatchImage;
    /**
     * @var SwatchCollectionFactory
     */
    protected $swatchCollectionFactory;

    /**
     * @var Repository
     */
    protected $attributeRepository;
    /**
     * @var \Magento\Catalog\Helper\ImageFactory
     */
    protected $imageHelperFactory;
    /**
     * @var \Magento\Swatches\Helper\Data
     */
    protected $swatchHelper;
    /**
     * @var \Magento\Catalog\Model\Product\Visibility
     */
    protected $productVisibility;
    /**
     * @var \Magento\Catalog\Model\Product\Attribute\Source\Status
     */
    protected $productStatus;

    /**
     * AttributeRelated constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param ImageBuilder $_imageBuilder
     * @param Registry $registry
     * @param CollectionFactory $productCollectionFactory
     * @param Stock $stockFilter
     * @param Media $swatchHelperMedia
     * @param Repository $attributeRepository
     * @param SwatchCollectionFactory $swatchCollectionFactory
     * @param Template\Context $context
     * @param \Magento\Swatches\Helper\Data $swatchHelper
     * @param Image $imageHelper
     * @param \Magento\Catalog\Helper\ImageFactory $imageHelperFactory
     * @param array $data
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ImageBuilder $_imageBuilder,
        Registry $registry,
        CollectionFactory $productCollectionFactory,
        Stock $stockFilter,
        Media $swatchHelperMedia,
        Repository $attributeRepository,
        SwatchCollectionFactory $swatchCollectionFactory,
        Template\Context $context,
        \Magento\Swatches\Helper\Data $swatchHelper,
        Image $imageHelper,
        \Magento\Catalog\Helper\ImageFactory $imageHelperFactory,
        \Magento\Catalog\Model\Product\Attribute\Source\Status $productStatus,
        \Magento\Catalog\Model\Product\Visibility $productVisibility,
        array $data = []
    )
    {

        $this->_scopeConfig = $scopeConfig;
        $this->imageBuilder = $_imageBuilder;
        $this->registry = $registry;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->stockFilter = $stockFilter;
        $this->swatchHelperMedia = $swatchHelperMedia;
        $this->attributeRepository = $attributeRepository;
        $this->swatchCollectionFactory = $swatchCollectionFactory;
        $this->imageHelper = $imageHelper;
        $this->swatchHelper = $swatchHelper;
        $this->imageHelperFactory = $imageHelperFactory;
        $this->productStatus = $productStatus;
        $this->productVisibility = $productVisibility;
        parent::__construct($context, $data);
    }

    /**
     * @return mixed
     */
    public function isAttributeRelatedEnabled()
    {
        return $this->_scopeConfig->getValue(
            self::ENABLED_ATTRIBUTE_RELATED_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function isListingAttributeRelatedEnabled()
    {
        return $this->_scopeConfig->getValue(
            self::ENABLED_ATTRIBUTE_RELATED_LISTING_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getAttributeLabel()
    {
        return $this->_scopeConfig->getValue(
            self::ATTRIBUTE_LABEL_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return \Magento\Catalog\Model\ResourceModel\Product\Collection
     * @throws NoSuchEntityException
     */
    public function getAttributeRelatedCollection($product)
    {
        if(!isset($product)){
            $product = $this->getCurrentProduct();
        }
        $collection = $this->productCollectionFactory->create()
            ->addAttributeToSelect('*')
            ->addAttributeToFilter($this->getRelationshipAttribute(), $this->getProductRelationAttributeValue($product))
            ->addAttributeToSelect('name')
            ->addAttributeToFilter('type_id', ['in' => $this->getAllowedProductTypes()])
            ->addAttributeToFilter('status', ['in' => $this->productStatus->getVisibleStatusIds()])
            ->addAttributeToFilter('visibility', ['in' => $this->productVisibility->getVisibleInSiteIds()])
            ->setPageSize($this->getProductLimit())
            ->setOrder('sku')
            ->addStoreFilter();

        if ($this->hideOutOfStock()) {
            $this->stockFilter->addInStockFilterToCollection($collection);
        }

        $swatchAttribute = $this->getSwatchAttribute();
        foreach ($collection->getItems() as $item) {
            $item->setData('fb_attribute_related_swatch', $this->lookUpAttribute($item, $item->getData($swatchAttribute)));
        }

        return $collection;
    }

    /**
     * @return mixed
     */

    public function getRelationshipAttribute()
    {
        return $this->_scopeConfig->getValue(
            self::ATTRIBUTE_RELATION_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getProductRelationAttributeValue($product)
    {
        return $product->getData($this->getRelationshipAttribute());
    }

    /**
     * Set Current Product From Registry
     */
    public function getCurrentProduct()
    {
        if ($this->currentProduct) {
            return $this->currentProduct;
        }

        return $this->currentProduct = $this->registry->registry('current_product');
    }

    /**
     * @return mixed
     */
    public function getAllowedProductTypes()
    {
        return $this->_scopeConfig->getValue(
            self::ALLOWED_PRODUCT_TYPE_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return int
     */
    public function getProductLimit()
    {
        return (int)$this->_scopeConfig->getValue(
            self::PRODUCT_LIMIT_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return  bool
     */
    public function hideOutOfStock()
    {
        return $this->_scopeConfig->getValue(
            self::HIDE_OUT_OF_STOCK_PATH,
            ScopeInterface::SCOPE_STORE);

    }

    /**
     * @return mixed
     */
    public function getSwatchAttribute()
    {
        return $this->_scopeConfig->getValue(
            self::SWATCH_ATTRIBUTE_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @param $attributeValue
     * @return array|mixed
     * @throws NoSuchEntityException
     */
    public function lookUpAttribute($product, $attributeValue)
    {
        if ($attributeValue === null) {
            return ['type' => 'image', 'value' => '#'];
        }

        if (empty($this->lookupWatch)) {

            if (!$this->defaultSwatchImage) {
                $this->defaultSwatchImage = $this->imageHelper->getDefaultPlaceholderUrl('swatch_image');
            }

            if (!$this->defaultSwatchThumb) {
                $this->defaultSwatchThumb = $this->imageHelper->getDefaultPlaceholderUrl('swatch_thumb');
            }

            $swatchMediaUrl = $this->swatchHelperMedia->getSwatchMediaUrl() . '/';

            foreach ($this->swatchCollectionFactory->create()->getItems() as $swatch) {

                $value = $swatch->getValue();

                switch ($swatch->getType()) {

                    case Swatch::SWATCH_TYPE_TEXTUAL:
                        $value = $value ?: 'Not set';
                        $type = 'text';
                        break;
                    case Swatch::SWATCH_TYPE_VISUAL_IMAGE:
                        $value = $value ? $swatchMediaUrl . $value : null;
                        $type = 'image';
                        break;
                    case Swatch::SWATCH_TYPE_VISUAL_COLOR:
                        $value = $value ?: '#050505';
                        $type = 'color';
                        break;
                    case  Swatch::SWATCH_TYPE_EMPTY;
                    default:
                        $type = 'image';
                }

                $this->lookupWatch[$swatch->getOptionId()] = ['type' => $type, 'value' => $value];
            }
        }

        if(!isset($this->lookupWatch[$attributeValue])) {
            return ['type' => 'image', 'value' => '#'];
        }

        $output = $this->lookupWatch[$attributeValue];
        $attribute = $this->attributeRepository->get($this->getSwatchAttribute());

        /* Workaround to force populateAdditionalDataEavAttribute call */
        if (!$this->swatchHelper->isVisualSwatch($attribute)) {
            return $output;
        }

        if (empty($output['value'])) {
            if ($attribute->getData('use_product_image_for_swatch')) {
                $output['value'] = $this->imageHelperFactory->create()->init($product, 'product_swatch_image_large')->getUrl();
            } else {
                $output['value'] = $this->defaultSwatchImage;
            }
        }

        return $output;
    }
}
